#! /usr/bin/perl
#
# man2html.pl Version 1.1
# Copyright 1997, 1998 by Richard Dawe
#
# This software is distributed under the terms of the GNU General Public
# License, which should have been distributed with this file as LICENSE. Please
# read the notice below.
#
# ---
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
#
# ---
#
# Description:
#
# This program uses the 'man' command to generate text output which is then
# converted into HTML using the PRE pre-formatted tags. Any references like
# "socket(2)" are converted into hyperlinks.
#
# The program must be run from the directory below the 'man' tree, i.e. the
# man pages will be in the relative paths man/man1, man/man2, etc. The pages
# will be put into a similar tree with paths like html/html1, etc.
#
# The program also generates an alphabetic index file in the directory below
# the man tree called 'index.html' which has links to all the man pages
# converted.

# Copyright message
$COPYRIGHT_TEXT = 'man2html.pl Version 1.1, Copyright 1997, 1998 by Richard Dawe';

$COPYRIGHT_MESSAGE = <<EOT;
$COPYRIGHT_TEXT

This program is distributed under the terms of the GNU Public License. Please
read the file LICENSE that should have been distributed with this program.

EOT

# Translation information
$TRANSLATION_MESSAGE = <<EOT;
<HTML>

<HEAD>
<TITLE>man2html Translation Issues</TITLE>
</HEAD>

<BODY>
<H1>man2html Translation Issues</H1>

man2html has some problems translating 'man' pages to HTML. These are:

<UL>
<LI>&lt; and &gt; cannot be translated properly, and are translated to (lt)
and (gt) respectively.
</UL><P>

<HR>
<I>$COPYRIGHT_TEXT</I>
</BODY>

</HTML>
EOT

$TRANSLATION_LINK = <<EOT;
<HR>
<I>Please look at the <A HREF=\"../trans.html\">man2html translation issues</A>.
<BR>$COPYRIGHT_TEXT</I>
EOT

# Help message
$HELP_MESSAGE = <<EOT;
man2html is a man page to HTML converter. It does this using a brute-force
approach - it *does not* understand the 'nroff' format of man pages. The
output is generated by filtering and search-and-replace. There are also
problems converting less-than and greater-than signs.

The program expects the man pages to be in a directory structure like
man/man1, man/man2, etc. It creates directories like html/html1 (i.e.
analogous to the man directory names) with HTML-converted man pages in. The
program should be run from the directory in which the root man directory is
situated, e.g. /usr (e.g. /usr/man as the man root, /usr/html as the
HTML-converted man root).

If the '-htm' option is used, output files will have the extension ".htm"
rather than ".html".
EOT

$INDEX_FILE = "index";          # Index filename stem
$HTML_EXTENSION = ".html";      # Long extension by default

# Print the copyright message
print $COPYRIGHT_MESSAGE;

# Parse the command line arguments
for ($i = 0; $i <= $#ARGV; $i++) {
    # Help?
    if ( ($ARGV[$i] eq '-h') || ($ARGV[$i] eq '--help') || ($ARGV[$i] eq '-?') )
    {
        print $HELP_MESSAGE;
        exit;
    }

    # Index filename
    if ($ARGV[$i] =~ m/$\-idx\:/) {
        @temp = split(/\:/, $ARGV[$i]);
        $INDEX_FILE = $temp[1];
    }

    # .htm rather than .html?
    if ($ARGV[$i] eq '-htm') { $HTML_EXTENSION = ".htm"; }
}

# Update $TRANSLATION_LINK
$TRANSLATION_LINK =~ s/\.html/$HTML_EXTENSION/g;

# Pages to put in index - format: caption, URL
@INDEXED = ();

# Check that the man directory exists
die "No man directory in current directory" if (! -e 'man');
die "man is not a directory" if (! -d 'man');

# Create the necessary html directories
if (! -e 'html')
{
    mkdir 'html', umask;
    print "Created \'html\' directory\n";
}

# Valid man<x> dirs, e.g. man1 - these two must be consistent for this program
# to work properly
@MANDIRS = (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 'o', 'n', 'l', 'p');
$MANDIRS_REGEXP = '0-9onlp';

# Recurse into these directories
chdir 'man';

for ($i = 0; $i <= $#MANDIRS; $i++)
{
    # Go to next one if this one doesn't exist
    if (! -e 'man' . $MANDIRS[$i])
    {
        print 'No man pages in section ' . $MANDIRS[$i] . "\n";
        next;
    }
    else
    { print 'Converting section ' . $MANDIRS[$i] . "\n"; }

    # Create html dir if necessary
    if (! -e '../html/html' . $MANDIRS[$i])
    {
        mkdir '../html/html' . $MANDIRS[$i], umask;
        print 'Created \'html/html' . $MANDIRS[$i] . "\' directory\n";
    }

    # Find all the '.<section>' files & request them via man & save as HTML in
    # the HTML directory
    opendir(DIR, 'man' . $MANDIRS[$i]) || die 'Unable to open directory man' . $MANDIRS[$i];
    @FILES = readdir(DIR);
    closedir(DIR);

    for ($j = 0; $j <= $#FILES; $j++)
    {
        if ( ($FILES[$j] eq '.') || ($FILES[$j] eq '..') ) { next; }

        $FILES[$j] = lc($FILES[$j]);        # Lowercase it for links
        $FILES[$j] =~ s/$\.$MANDIRS[$i]//;  # Remove number extension

        $MANPAGENAME = $FILES[$j];
        #$MANPAGENAME =~ s/$\.$MANDIRS[$i]//;   # Done already

        warn "Executing: man - $MANDIRS[$i] $MANPAGENAME";
        $MANPAGE = `man - $MANDIRS[$i] $MANPAGENAME`;

        # Nuke weird control characters
        $MANPAGE =~ s/.\x08//g;

        # Escape characters
        $MANPAGE =~ s/</(lt)/g;
        $MANPAGE =~ s/>/(gt)/g;

        # Generate links
        $MANPAGE =~ s/(\w+)\(([$MANDIRS_REGEXP])\)/\<A HREF\=\"\L..\/html$2\/$1$HTML_EXTENSION\E\"\>$1($2)\<\/A\>/g;

        # Output it
        open(OUTPUT, ">../html/html" . $MANDIRS[$i] . "/" . $FILES[$j] . $HTML_EXTENSION)
            || die "Unable to open output file";

        print OUTPUT "<HTML>\n";
        print OUTPUT "<HEAD>\n<TITLE>$MANPAGENAME</TITLE>\n</HEAD>\n";
        print OUTPUT "<BODY>\n";
        print OUTPUT "<PRE>$MANPAGE</PRE>\n";
        print OUTPUT $TRANSLATION_LINK;
        print OUTPUT "</BODY>\n";
        print OUTPUT "</HTML>\n";

        close OUTPUT;

        # Put entry in index
        push(@INDEXED, ($MANPAGENAME."(".$MANDIRS[$i]."):html".$MANDIRS[$i]."/".$FILES[$j].$HTML_EXTENSION));

        # Status message
        print "Converted $FILES[$j] man page\n";
    }
}

# Change back
chdir('..');

# Create an index file
open(INDEX, ">html/$INDEX_FILE$HTML_EXTENSION") || die 'Unable to create index file';
print INDEX "<HTML>\n";
print INDEX "<HEAD>\n<TITLE>Man Page Index</TITLE>\n</HEAD>\n";
print INDEX "<BODY>\n<H1>Man Page Index</H1>\n";

@INDEXED = sort(@INDEXED);
$LETTER = '';               # Used to generate letter index at top

for ($i = 'A'; $i lt 'Z'; $i++)
{ print INDEX "<A HREF=\"#$i\">$i</A> "; }
print INDEX "<A HREF=\"#Z\">Z</A> ";
print INDEX "<P>\n";

for ($i = 0; $i <= $#INDEXED; $i++)
{   
    @temp = split(/:/, $INDEXED[$i]);

    # Do we need a new letter from the alphabet?
    if (uc(substr($temp[0], 0, 1)) ne $LETTER)
    {
        $LETTER = uc(substr($temp[0], 0, 1));
        print INDEX "<H2><A NAME=\"$LETTER\">$LETTER</A></H2>\n";
    }

    # Print the index entry
    print INDEX "<A HREF=\"".lc($temp[1])."\">".$temp[0]."</A><BR>\n";
}

# Change the relative link for the translation issues file & output
$TRANSLATION_LINK =~ s/\.\.\/trans$HTML_EXTENSION/trans$HTML_EXTENSION/g;
print INDEX "<P>$TRANSLATION_LINK";

print INDEX "</BODY>\n</HTML>\n";
close INDEX;

# Create the translation issues file
open(TRANS, ">html/trans$HTML_EXTENSION") || die 'Unable to create translation information file';
print TRANS $TRANSLATION_MESSAGE;
close TRANS;

# Finished
__END__
